var __defProp = Object.defineProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// src/index.ts
import { EntityEquippableComponent as EntityEquippableComponent5, EquipmentSlot as EquipmentSlot5, Player as Player5, system, world as world2 } from "@minecraft/server";

// src/lib/book.ts
var ExperienceBook = class {
  savedExperience;
  itemStack;
  owner;
  constructor(itemStack, owner) {
    this.itemStack = itemStack;
    this.deserialize();
    this.savedExperience = this.savedExperience ?? 0;
    if (!this.owner)
      this.owner = owner;
  }
  deserialize() {
    this.savedExperience = this.itemStack.getDynamicProperty("true:xp_book");
    this.owner = this.itemStack.getDynamicProperty("true:owner");
  }
  serialize() {
    this.itemStack.setDynamicProperty("true:xp_book", this.savedExperience);
    this.itemStack.setDynamicProperty("true:owner", this.owner ?? "unknown");
  }
  addXp(amount) {
    const resultExperience = this.savedExperience + amount;
    this.savedExperience = Math.max(0, resultExperience);
    this.itemStack.setLore(this.lore);
    this.serialize();
    return this;
  }
  get lore() {
    return [`\xA7r\xA72Saved XP: ${this.savedExperience}`, `\xA7r\xA79Owner: \xA7r\xA73${this.owner}`];
  }
};

// src/lib/forms/index.ts
var forms_exports = {};
__export(forms_exports, {
  FormRegistry: () => FormRegistry,
  MainForm: () => MainForm,
  RetrieveForm: () => RetrieveForm,
  SaveForm: () => SaveForm,
  TransferForm: () => TransferForm
});

// src/lib/forms/transfer.ts
import { EntityEquippableComponent, EquipmentSlot, world } from "@minecraft/server";
import { ModalFormData } from "@minecraft/server-ui";

// src/lib/forms/form.ts
var Form = class {
  static form;
  static identifier;
  static show(player) {
    if (!this.form)
      this.builder();
    this.form.show(player).then((formResponse) => {
      this.handler(player, formResponse);
    });
  }
  static builder(..._args) {
    return this;
  }
  static handler(_sender, _formResponse) {
  }
};

// src/lib/forms/transfer.ts
var TransferForm = class extends Form {
  static identifier = "transfer";
  static playerList = [];
  static builder(savedExperience) {
    this.playerList = world.getAllPlayers().map((player) => player.name);
    this.form = new ModalFormData().title({ translate: "xpbook.forms.transfer.title" }).dropdown(
      { translate: "xpbook.forms.transfer.playerlist" },
      this.playerList
    ).slider({ translate: "xpbook.forms.transfer.xp_slider" }, 0, savedExperience, 2, 0);
    return this;
  }
  static handler(sender, formResponse) {
    const { formValues, canceled } = formResponse;
    if (canceled || !formValues)
      return;
    const [selectedPlayerIdx, selectedAmount] = formValues;
    const selectedPlayer = world.getAllPlayers().find((player) => player.name === this.playerList[selectedPlayerIdx]);
    const playerEquipment = sender.getComponent(EntityEquippableComponent.componentId);
    const mainHand = playerEquipment.getEquipment(EquipmentSlot.Mainhand);
    if (!selectedPlayer)
      return sender.sendMessage({
        translate: "xpbook.forms.transfer.player_not_found"
      });
    else if (selectedPlayer.id == sender.id) {
      sender.sendMessage({
        translate: "xpbook.forms.transfer.self_transfer"
      });
      return;
    }
    if (!mainHand)
      return;
    sender.sendMessage({
      translate: "xpbook.messages.transfer.success",
      with: [selectedAmount.toString()]
    });
    selectedPlayer.sendMessage({
      translate: "xpbook.messages.transfered.success",
      with: [selectedAmount.toString()]
    });
    new ExperienceBook(mainHand).addXp(-selectedAmount);
    playerEquipment.setEquipment(EquipmentSlot.Mainhand, mainHand);
    selectedPlayer.addLevels(selectedAmount);
  }
};

// src/lib/forms/save.ts
import { ModalFormData as ModalFormData2 } from "@minecraft/server-ui";
import { EntityEquippableComponent as EntityEquippableComponent2, EquipmentSlot as EquipmentSlot2 } from "@minecraft/server";
var SaveForm = class extends Form {
  static identifier = "save";
  static builder(playerExperience) {
    this.form = new ModalFormData2().title("XP Book - Save XP").slider({ translate: "forms.generic.xp_amount" }, 0, playerExperience, 1);
    return this;
  }
  static show(player) {
    if (player.level < 10)
      return player.sendMessage({
        translate: "xpbook.messages.save.insufficient_level"
      });
    super.show(player);
  }
  static handler(sender, formResponse) {
    const { canceled, formValues } = formResponse;
    if (canceled || !formValues)
      return;
    const [xpLevels] = formValues;
    const playerEquipment = sender.getComponent(EntityEquippableComponent2.componentId);
    const mainHand = playerEquipment.getEquipment(EquipmentSlot2.Mainhand);
    if (!mainHand)
      return;
    sender.sendMessage({
      translate: "xpbook.messages.save.success",
      with: [xpLevels.toString()]
    });
    sender.addLevels(-xpLevels);
    new ExperienceBook(mainHand).addXp(xpLevels);
    playerEquipment.setEquipment(EquipmentSlot2.Mainhand, mainHand);
  }
};

// src/lib/forms/main.ts
import { ActionFormData } from "@minecraft/server-ui";
import { EntityEquippableComponent as EntityEquippableComponent3, EquipmentSlot as EquipmentSlot3 } from "@minecraft/server";

// src/lib/forms/registry.ts
var FormRegistry = class _FormRegistry {
  static forms = /* @__PURE__ */ new Map();
  static registerForm(form) {
    _FormRegistry.forms.set(form.identifier, form);
  }
  static getForm(id) {
    return _FormRegistry.forms.get(id);
  }
};

// src/lib/forms/main.ts
var MainForm = class extends Form {
  static identifier = "main";
  static builder(toggle) {
    this.form = new ActionFormData().title("xpbook.title_panel").body(`\xA79Soul Bind: ${toggle ? "\xA72Active" : "\xA74Inactive"}`).button({ translate: "xpbook.save.button" }, "textures/ui/experience_book/save_xp.png").button({ translate: "xpbook.retrieve.button" }, "textures/ui/experience_book/get_xp.png").button({ translate: "xpbook.saveall.button" }, "textures/ui/experience_book/save_all.png").button({ translate: "xpbook.retrieveall.button" }, "textures/ui/experience_book/get_all_xp.png").button({ translate: "xpbook.tranfer.button" }, "textures/ui/experience_book/transfer_xp.png").button({ translate: "xpbook.soulbind.button" }, "textures/ui/experience_book/soul_bind.png");
        return this;
  }
  static handler(sender, formResponse) {
    const { canceled, selection } = formResponse;
    if (canceled || selection === void 0)
      return;
    const buttonMap = /* @__PURE__ */ new Map([
      [0, "save"],
      [1, "retrieve"],
      [4, "transfer"]
    ]);
    const selectedFormId = buttonMap.get(selection);
    const playerEquipment = sender.getComponent(EntityEquippableComponent3.componentId);
    const mainHand = playerEquipment.getEquipment(EquipmentSlot3.Mainhand);
    if (!mainHand)
      return;
    if (!selectedFormId) {
      const experienceBook = new ExperienceBook(mainHand);
      switch (selection) {
        case 2: {
          experienceBook.addXp(sender.level);
          sender.addLevels(-sender.level);
          sender.sendMessage({ translate: "xpbook.saveall.success" });
          break;
        }
        case 3: {
          sender.addLevels(experienceBook.savedExperience);
          experienceBook.addXp(-experienceBook.savedExperience);
          sender.sendMessage({ translate: "xpbook.retrieveall.success" });
          break;
        }
        case 5: {
          mainHand.keepOnDeath = !mainHand.keepOnDeath;
          break;
        }
      }
      playerEquipment.setEquipment(EquipmentSlot3.Mainhand, mainHand);
      return;
    }
    const selectedForm = FormRegistry.getForm(selectedFormId);
    if (!selectedForm)
      return;
    selectedForm.builder(selectedFormId == "transfer" || selectedFormId == "retrieve" ? new ExperienceBook(mainHand).savedExperience : sender.level).show(sender);
  }
};

// src/lib/forms/retrieve.ts
import { ModalFormData as ModalFormData3 } from "@minecraft/server-ui";
import { EntityEquippableComponent as EntityEquippableComponent4, EquipmentSlot as EquipmentSlot4 } from "@minecraft/server";
var RetrieveForm = class extends Form {
  static identifier = "retrieve";
  static builder(savedExperience) {
    this.form = new ModalFormData3().title("XP Book - Retrieve XP").slider({ translate: "forms.generic.xp_amount" }, 0, savedExperience, 1);
    return this;
  }
  static handler(sender, formResponse) {
    const { formValues, canceled } = formResponse;
    if (canceled || !formValues)
      return;
    const [xpLevels] = formValues;
    const playerEquipment = sender.getComponent(EntityEquippableComponent4.componentId);
    const mainHand = playerEquipment.getEquipment(EquipmentSlot4.Mainhand);
    if (!mainHand)
      return;
    sender.sendMessage({
      translate: "xpbook.messages.retrieve.success",
      with: [xpLevels.toString()]
    });
    sender.addLevels(xpLevels);
    new ExperienceBook(mainHand).addXp(-xpLevels);
    playerEquipment.setEquipment(EquipmentSlot4.Mainhand, mainHand);
  }
};

// src/index.ts
world2.beforeEvents.itemUse.subscribe((event) => {
  const { itemStack, source } = event;
  if (itemStack.typeId !== "true:xp_book" || !(source instanceof Player5))
    return;
  const mainForm = FormRegistry.getForm("main");
  if (!mainForm)
    return console.warn("Main form aint registered");
  const playerEquipment = source.getComponent(EntityEquippableComponent5.componentId);
  system.run(() => {
    new ExperienceBook(itemStack, source.name).addXp(0).serialize();
    const mainHand = playerEquipment.getEquipment(EquipmentSlot5.Mainhand);
    if (!mainHand || mainHand.typeId !== itemStack.typeId)
      return;
    playerEquipment.setEquipment(EquipmentSlot5.Mainhand, itemStack);
    mainForm.builder(itemStack.keepOnDeath).show(source);
  });
});
for (const key in forms_exports) {
  const form = forms_exports[key];
  if (!form.identifier)
    continue;
  FormRegistry.registerForm(form);
}
